//******************************************************************************************
//* C-code source file "ILI9325.c" with function calls for 2.4 inch TFT with 320x240 pixel
//*
//* Created: 17.09.2014 16:56:55
//* Last modification: 11.12.2014
//*
//* Author: Dirk Uffmann (uffmann@arcor.de)
//*
//* This program should run on an Arduino Mega 2560 R3 board with an Lseeduino TFT
//* shield V1.3 and 2.4 inch TFT V2.1 with 320x240 pixel.
//* Intention is to test an OV7670 Camera module by Alientek V2.2 with FIFO AL422B.
//* For debugging purpose a text LCD with I2C bus adapter is connected via TWI.
//*
//* The hardware and software is documented in an article provided in the internet
//* magazine ft:pedia 04/2014 by www.ftcommunity.de in german language
//*
//* Title of the article: "Einfache Ziffernerkennung mit einer CMOS-Kamera am AVR-
//* Controller am Beispiel eines ft-Kreditkartenleser-Modells"
//*
//* This software is provided as is without any warranty, guarantee or liability.
//* The author takes no responsibility for consequences or damage caused by the usage
//* of this software.
//*
//******************************************************************************************

#include "ILI9325D.h"

void
TFT_write_index(uint8_t index)
{
	//	switch Camera outputs off
	PORTF |= _BV(PF3);	// Camera OE set high (inactive)

	//	switch Data Port A to output and low (for sending "0x00" as first byte of register index)
	PORTA = 0;
	DDRA = 0xFF;

	//	clear TFT CS chip select (PG1) and TFT Register select (PD7)
	PORTG &= ~(_BV(PG1));
	PORTD &= ~(_BV(PD7));
	
	//	clear TFT write (PG2)
	PORTG &= ~(_BV(PG2));

	//	set TFT write
	PORTG |= _BV(PG2);

	//	send register index byte
	PORTA = index;

	//	clear TFT write (PG2)
	PORTG &= ~(_BV(PG2));

	//	set TFT write
	PORTG |= _BV(PG2);

	//	set TFT register select	RS
	PORTD |= _BV(PD7);

	//	switch Data Port A to input and no pull-ups
	PORTA = 0;
	DDRA = 0;
}

void
TFT_write_Reg(uint8_t index, uint8_t reg_val_high, uint8_t reg_val_low)
{
	//	switch Camera outputs off
	PORTF |= _BV(PF3);	// Camera OE set high (inactive)

	//	switch Data Port A to output and low (for sending "0x00" as first byte of register index)
	PORTA = 0;
	DDRA = 0xFF;

	//	clear TFT CS chip select (PG1) and TFT Register select (PD7)
	PORTG &= ~(_BV(PG1));
	PORTD &= ~(_BV(PD7));
	
	//	clear TFT write (PG2), ie. write strobe
	PORTG &= ~(_BV(PG2));

	//	set TFT write
	PORTG |= _BV(PG2);

	//	send register index byte
	PORTA = index;

	//	clear TFT write (PG2)
	PORTG &= ~(_BV(PG2));

	//	set TFT write
	PORTG |= _BV(PG2);

	//	set TFT register select	RS
	PORTD |= _BV(PD7);

	PORTA = reg_val_high;

	//	clear TFT write (PG2)
	PORTG &= ~(_BV(PG2));

	//	set TFT write
	PORTG |= _BV(PG2);

	PORTA = reg_val_low;

	//	clear TFT write (PG2)
	PORTG &= ~(_BV(PG2));

	//	set TFT write
	PORTG |= _BV(PG2);
	
	//	set TFT chip select
	PORTG |= _BV(PG1);

	//	switch Data Port A to input and no pull-ups
	PORTA = 0;
	DDRA = 0;
}

void
TFT_init(void)
{
	//	Reset ILI9325D TFT color display 320x240 pixel
	PORTG &= ~(_BV(PG0)); //clear TFT reset line
	lcd_wait_ms(1);
	PORTG |= _BV(PG0); //set TFT reset line

	//	lcd_wait_ms(1); // Dis-charge capacitor power voltage

	// After Reset: Display off
	//	GON = 0; DTE = 0; D[1:0] = 00
	//	This is default after reset and must not expilicitly
	//	written to the register
	TFT_write_Reg(0x07, 0x00, 0x00); //clear display control 1

	//*************Power On sequence ****************//

	//********Power supply initial setting***********//
	//	Set VC[2:0], VRH[3:0], VCM[5:0], VDV[4:0] to their target setting
	//	Default: BT[2:0]=000

	TFT_write_Reg(0x11, 0x00, 0x07);	// DC1[2:0], DC0[2:0], VC[2:0]
	TFT_write_Reg(0x12, 0x00, 0x0D);	// Set VREG1OUT voltage to VCI x 1.85 = 4.625V
	TFT_write_Reg(0x13, 0x12, 0x00);	// VDV[4:0] for VCOM amplitude
	TFT_write_Reg(0x29, 0x00, 0x0A);	// VCM[5:0] for VCOMH
	// VCOMH = VREG1OUT x 0.735
	//	wait 50 ms
	lcd_wait_ms(50); // Dis-charge capacitor power voltage

	//*******Power supply operation setting**********//
	//	Set BT[2:0], Set DC1[2:0], DC0[2:0], Set AP[2:0], APE=1
	TFT_write_Reg(0x10, 0x06, 0x90); // SAP=0, Set BT[3:0], APE, AP[2:0], SLP, STB
	TFT_write_Reg(0x11, 0x02, 0x27); // Set DC1[2:0], DC0[2:0], VC[2:0]

	//	wait 80ms
	lcd_wait_ms(80); // step-up circuit stabilization

	//******Set all other registers******************//
	TFT_write_Reg(0xE5, 0x78, 0xF0); // set SRAM internal timing

	TFT_write_Reg(0x01, 0x00, 0x00); // set Driver Output Control, SS = 0
	TFT_write_Reg(0x02, 0x00, 0x20); // set line inversion (better quality)
	//	TFT_write_Reg(0x03, 0x10, 0x38); // set GRAM write direction to increment and make vertical address counter update (320 pixel/line).
	TFT_write_Reg(0x03, 0x10, 0xA8); // modified to mirror image on vertical axis and to move the GRAM start Address according to increment/decrement
	TFT_write_Reg(0x04, 0x00, 0x00); // Resize register
	TFT_write_Reg(0x08, 0x02, 0x07); // set the back porch and front porch
	TFT_write_Reg(0x09, 0x00, 0x00); // set non-display area refresh cycle ISC[3:0]
	TFT_write_Reg(0x0A, 0x00, 0x00); // FMARK function
	TFT_write_Reg(0x0C, 0x00, 0x00); // RGB interface setting
	TFT_write_Reg(0x0D, 0x00, 0x00); // Frame marker Position
	TFT_write_Reg(0x0F, 0x00, 0x00); // RGB interface polarity
	TFT_write_Reg(0x20, 0x00, 0x00); // GRAM horizontal Address
	TFT_write_Reg(0x21, 0x00, 0x00); // GRAM Vertical Address
	TFT_write_Reg(0x2B, 0x00, 0x0B); // Set Frame Rate to 72 FPS
	// ----------- Adjust the Gamma Curve ----------//
	TFT_write_Reg(0x30, 0x00, 0x00);
	TFT_write_Reg(0x31, 0x04, 0x04);
	TFT_write_Reg(0x32, 0x00, 0x03);
	TFT_write_Reg(0x35, 0x04, 0x05);
	TFT_write_Reg(0x36, 0x08, 0x08);
	TFT_write_Reg(0x37, 0x04, 0x07);
	TFT_write_Reg(0x38, 0x03, 0x03);
	TFT_write_Reg(0x39, 0x07, 0x07);
	TFT_write_Reg(0x3C, 0x05, 0x04);
	TFT_write_Reg(0x3D, 0x08, 0x08);
	//------------------ Set GRAM area ---------------//
	TFT_write_Reg(0x50, 0x00, 0x00); // Horizontal GRAM Start Address
	TFT_write_Reg(0x51, 0x00, 0xEF); // Horizontal GRAM End Address = 239 (QQVGA(160x120) image height=120, provide space for two images)
	TFT_write_Reg(0x52, 0x00, 0x00); // Vertical GRAM Start Address
	TFT_write_Reg(0x53, 0x00, 0x9F); // Vertical GRAM End Address = 159 (QQVGA(160x120) images width = 160)
	TFT_write_Reg(0x60, 0x27, 0x00); // Gate Scan Lines = 320 default
	TFT_write_Reg(0x61, 0x00, 0x01); // NDL,VLE, REV
	TFT_write_Reg(0x6A, 0x00, 0x00); // set scrolling line
	//-------------- Partial Display Control ---------//
	TFT_write_Reg(0x80, 0x00, 0x00);
	TFT_write_Reg(0x81, 0x00, 0x00);
	TFT_write_Reg(0x82, 0x00, 0x9F);
	TFT_write_Reg(0x83, 0x00, 0xA0);
	TFT_write_Reg(0x84, 0x00, 0xA0);
	TFT_write_Reg(0x85, 0x01, 0x3F);
	/*
	Register 0x07 = Display Control 1 -> 0x30xx
	Register 0x60 = Driver Output Control 2 -> 0x27xx
	Register 0x80 = Partial Image 1 Display Position -> 0x0000
	Register 0x81 = Partial Image 1 Area (Start Line) -> 0x0000
	Register 0x82 = Partial Image 1 Area (End Line) -> 0x009F (160 Zeilen, geflippt)
	Register 0x83 = Partial Image 2 Display Position -> 0x00A0
	Register 0x84 = Partial Image 2 Area (Start Line) -> 0x00A0
	Register 0x85 = Partial Image 2 Area (End Line) -> 0x013F (weitere 160 Zeilen)
	*/

	//-------------- Panel Control -------------------//
	TFT_write_Reg(0x90, 0x00, 0x10);
	TFT_write_Reg(0x92, 0x00, 0x00);

	//******  Display on sequence  **********************//
	//	Set SAP=1
	//	GON = 0; DTE = 0; D[1:0] = 01
	TFT_write_Reg(0x10, 0x16, 0x90); // SAP=1, Set BT[3:0], APE, AP[2:0], SLP, STB
	TFT_write_Reg(0x07, 0x00, 0x01);

	//	Wait for 2 frames
	lcd_wait_ms(50);

	//	GON = 1; DTE = 0; D[1:0] = 01
	TFT_write_Reg(0x07, 0x00, 0x21);

	//	GON = 1; DTE = 0; D[1:0] = 11
	TFT_write_Reg(0x07, 0x00, 0x23);

	//	Wait for 2 frames
	lcd_wait_ms(50);

	//	GON = 1; DTE = 1; D[1:0] = 11
	//	TFT_write_Reg(0x07, 0x30, 0x33); // 262K color and display ON, two partial images enabled
	TFT_write_Reg(0x07, 0x01, 0x33); // 262K color and display ON, base image enabled
}

